/****************************************************************************
** Meta object code from reading C++ file 'canvasrenderer_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/qtcanvas3d-opensource-src-5.9.2/src/imports/qtcanvas3d/canvasrenderer_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'canvasrenderer_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtCanvas3D__CanvasRenderer_t {
    QByteArrayData data[13];
    char stringdata0[131];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtCanvas3D__CanvasRenderer_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtCanvas3D__CanvasRenderer_t qt_meta_stringdata_QtCanvas3D__CanvasRenderer = {
    {
QT_MOC_LITERAL(0, 0, 26), // "QtCanvas3D::CanvasRenderer"
QT_MOC_LITERAL(1, 27, 10), // "fpsChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 3), // "fps"
QT_MOC_LITERAL(4, 43, 12), // "textureReady"
QT_MOC_LITERAL(5, 56, 2), // "id"
QT_MOC_LITERAL(6, 59, 4), // "size"
QT_MOC_LITERAL(7, 64, 17), // "textureIdResolved"
QT_MOC_LITERAL(8, 82, 11), // "QQuickItem*"
QT_MOC_LITERAL(9, 94, 4), // "item"
QT_MOC_LITERAL(10, 99, 8), // "shutDown"
QT_MOC_LITERAL(11, 108, 6), // "render"
QT_MOC_LITERAL(12, 115, 15) // "clearBackground"

    },
    "QtCanvas3D::CanvasRenderer\0fpsChanged\0"
    "\0fps\0textureReady\0id\0size\0textureIdResolved\0"
    "QQuickItem*\0item\0shutDown\0render\0"
    "clearBackground"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtCanvas3D__CanvasRenderer[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   44,    2, 0x06 /* Public */,
       4,    2,   47,    2, 0x06 /* Public */,
       7,    1,   52,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   55,    2, 0x0a /* Public */,
      11,    0,   56,    2, 0x0a /* Public */,
      12,    0,   57,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::UInt,    3,
    QMetaType::Void, QMetaType::Int, QMetaType::QSize,    5,    6,
    QMetaType::Void, 0x80000000 | 8,    9,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void QtCanvas3D::CanvasRenderer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        CanvasRenderer *_t = static_cast<CanvasRenderer *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->fpsChanged((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 1: _t->textureReady((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QSize(*)>(_a[2]))); break;
        case 2: _t->textureIdResolved((*reinterpret_cast< QQuickItem*(*)>(_a[1]))); break;
        case 3: _t->shutDown(); break;
        case 4: _t->render(); break;
        case 5: _t->clearBackground(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (CanvasRenderer::*_t)(uint );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&CanvasRenderer::fpsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (CanvasRenderer::*_t)(int , const QSize & );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&CanvasRenderer::textureReady)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (CanvasRenderer::*_t)(QQuickItem * );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&CanvasRenderer::textureIdResolved)) {
                *result = 2;
                return;
            }
        }
    }
}

const QMetaObject QtCanvas3D::CanvasRenderer::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_QtCanvas3D__CanvasRenderer.data,
      qt_meta_data_QtCanvas3D__CanvasRenderer,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *QtCanvas3D::CanvasRenderer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtCanvas3D::CanvasRenderer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtCanvas3D__CanvasRenderer.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QOpenGLFunctions"))
        return static_cast< QOpenGLFunctions*>(this);
    return QObject::qt_metacast(_clname);
}

int QtCanvas3D::CanvasRenderer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QtCanvas3D::CanvasRenderer::fpsChanged(uint _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QtCanvas3D::CanvasRenderer::textureReady(int _t1, const QSize & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QtCanvas3D::CanvasRenderer::textureIdResolved(QQuickItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
