/****************************************************************************
** Meta object code from reading C++ file 'glcommandqueue_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../src/qtcanvas3d-opensource-src-5.9.2/src/imports/qtcanvas3d/glcommandqueue_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'glcommandqueue_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtCanvas3D__CanvasGlCommandQueue_t {
    QByteArrayData data[145];
    char stringdata0[2371];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtCanvas3D__CanvasGlCommandQueue_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtCanvas3D__CanvasGlCommandQueue_t qt_meta_stringdata_QtCanvas3D__CanvasGlCommandQueue = {
    {
QT_MOC_LITERAL(0, 0, 32), // "QtCanvas3D::CanvasGlCommandQueue"
QT_MOC_LITERAL(1, 33, 9), // "queueFull"
QT_MOC_LITERAL(2, 43, 0), // ""
QT_MOC_LITERAL(3, 44, 11), // "GlCommandId"
QT_MOC_LITERAL(4, 56, 17), // "internalNoCommand"
QT_MOC_LITERAL(5, 74, 15), // "glActiveTexture"
QT_MOC_LITERAL(6, 90, 14), // "glAttachShader"
QT_MOC_LITERAL(7, 105, 20), // "glBindAttribLocation"
QT_MOC_LITERAL(8, 126, 12), // "glBindBuffer"
QT_MOC_LITERAL(9, 139, 17), // "glBindFramebuffer"
QT_MOC_LITERAL(10, 157, 18), // "glBindRenderbuffer"
QT_MOC_LITERAL(11, 176, 13), // "glBindTexture"
QT_MOC_LITERAL(12, 190, 12), // "glBlendColor"
QT_MOC_LITERAL(13, 203, 15), // "glBlendEquation"
QT_MOC_LITERAL(14, 219, 23), // "glBlendEquationSeparate"
QT_MOC_LITERAL(15, 243, 11), // "glBlendFunc"
QT_MOC_LITERAL(16, 255, 19), // "glBlendFuncSeparate"
QT_MOC_LITERAL(17, 275, 12), // "glBufferData"
QT_MOC_LITERAL(18, 288, 15), // "glBufferSubData"
QT_MOC_LITERAL(19, 304, 24), // "glCheckFramebufferStatus"
QT_MOC_LITERAL(20, 329, 7), // "glClear"
QT_MOC_LITERAL(21, 337, 12), // "glClearColor"
QT_MOC_LITERAL(22, 350, 13), // "glClearDepthf"
QT_MOC_LITERAL(23, 364, 14), // "glClearStencil"
QT_MOC_LITERAL(24, 379, 11), // "glColorMask"
QT_MOC_LITERAL(25, 391, 15), // "glCompileShader"
QT_MOC_LITERAL(26, 407, 22), // "glCompressedTexImage2D"
QT_MOC_LITERAL(27, 430, 25), // "glCompressedTexSubImage2D"
QT_MOC_LITERAL(28, 456, 16), // "glCopyTexImage2D"
QT_MOC_LITERAL(29, 473, 19), // "glCopyTexSubImage2D"
QT_MOC_LITERAL(30, 493, 15), // "glCreateProgram"
QT_MOC_LITERAL(31, 509, 14), // "glCreateShader"
QT_MOC_LITERAL(32, 524, 10), // "glCullFace"
QT_MOC_LITERAL(33, 535, 15), // "glDeleteBuffers"
QT_MOC_LITERAL(34, 551, 20), // "glDeleteFramebuffers"
QT_MOC_LITERAL(35, 572, 15), // "glDeleteProgram"
QT_MOC_LITERAL(36, 588, 21), // "glDeleteRenderbuffers"
QT_MOC_LITERAL(37, 610, 14), // "glDeleteShader"
QT_MOC_LITERAL(38, 625, 16), // "glDeleteTextures"
QT_MOC_LITERAL(39, 642, 11), // "glDepthFunc"
QT_MOC_LITERAL(40, 654, 11), // "glDepthMask"
QT_MOC_LITERAL(41, 666, 13), // "glDepthRangef"
QT_MOC_LITERAL(42, 680, 14), // "glDetachShader"
QT_MOC_LITERAL(43, 695, 9), // "glDisable"
QT_MOC_LITERAL(44, 705, 26), // "glDisableVertexAttribArray"
QT_MOC_LITERAL(45, 732, 12), // "glDrawArrays"
QT_MOC_LITERAL(46, 745, 14), // "glDrawElements"
QT_MOC_LITERAL(47, 760, 8), // "glEnable"
QT_MOC_LITERAL(48, 769, 25), // "glEnableVertexAttribArray"
QT_MOC_LITERAL(49, 795, 7), // "glFlush"
QT_MOC_LITERAL(50, 803, 8), // "glFinish"
QT_MOC_LITERAL(51, 812, 25), // "glFramebufferRenderbuffer"
QT_MOC_LITERAL(52, 838, 22), // "glFramebufferTexture2D"
QT_MOC_LITERAL(53, 861, 11), // "glFrontFace"
QT_MOC_LITERAL(54, 873, 12), // "glGenBuffers"
QT_MOC_LITERAL(55, 886, 16), // "glGenerateMipmap"
QT_MOC_LITERAL(56, 903, 17), // "glGenFramebuffers"
QT_MOC_LITERAL(57, 921, 18), // "glGenRenderbuffers"
QT_MOC_LITERAL(58, 940, 13), // "glGenTextures"
QT_MOC_LITERAL(59, 954, 17), // "glGetActiveAttrib"
QT_MOC_LITERAL(60, 972, 18), // "glGetActiveUniform"
QT_MOC_LITERAL(61, 991, 19), // "glGetAttribLocation"
QT_MOC_LITERAL(62, 1011, 13), // "glGetBooleanv"
QT_MOC_LITERAL(63, 1025, 22), // "glGetBufferParameteriv"
QT_MOC_LITERAL(64, 1048, 10), // "glGetError"
QT_MOC_LITERAL(65, 1059, 11), // "glGetFloatv"
QT_MOC_LITERAL(66, 1071, 37), // "glGetFramebufferAttachmentPar..."
QT_MOC_LITERAL(67, 1109, 13), // "glGetIntegerv"
QT_MOC_LITERAL(68, 1123, 19), // "glGetProgramInfoLog"
QT_MOC_LITERAL(69, 1143, 14), // "glGetProgramiv"
QT_MOC_LITERAL(70, 1158, 28), // "glGetRenderbufferParameteriv"
QT_MOC_LITERAL(71, 1187, 18), // "glGetShaderInfoLog"
QT_MOC_LITERAL(72, 1206, 13), // "glGetShaderiv"
QT_MOC_LITERAL(73, 1220, 26), // "glGetShaderPrecisionFormat"
QT_MOC_LITERAL(74, 1247, 11), // "glGetString"
QT_MOC_LITERAL(75, 1259, 19), // "glGetTexParameteriv"
QT_MOC_LITERAL(76, 1279, 14), // "glGetUniformfv"
QT_MOC_LITERAL(77, 1294, 14), // "glGetUniformiv"
QT_MOC_LITERAL(78, 1309, 20), // "glGetUniformLocation"
QT_MOC_LITERAL(79, 1330, 25), // "glGetVertexAttribPointerv"
QT_MOC_LITERAL(80, 1356, 19), // "glGetVertexAttribfv"
QT_MOC_LITERAL(81, 1376, 19), // "glGetVertexAttribiv"
QT_MOC_LITERAL(82, 1396, 6), // "glHint"
QT_MOC_LITERAL(83, 1403, 10), // "glIsBuffer"
QT_MOC_LITERAL(84, 1414, 11), // "glIsEnabled"
QT_MOC_LITERAL(85, 1426, 15), // "glIsFramebuffer"
QT_MOC_LITERAL(86, 1442, 11), // "glIsProgram"
QT_MOC_LITERAL(87, 1454, 16), // "glIsRenderbuffer"
QT_MOC_LITERAL(88, 1471, 10), // "glIsShader"
QT_MOC_LITERAL(89, 1482, 11), // "glIsTexture"
QT_MOC_LITERAL(90, 1494, 11), // "glLineWidth"
QT_MOC_LITERAL(91, 1506, 13), // "glLinkProgram"
QT_MOC_LITERAL(92, 1520, 13), // "glPixelStorei"
QT_MOC_LITERAL(93, 1534, 15), // "glPolygonOffset"
QT_MOC_LITERAL(94, 1550, 12), // "glReadPixels"
QT_MOC_LITERAL(95, 1563, 21), // "glRenderbufferStorage"
QT_MOC_LITERAL(96, 1585, 16), // "glSampleCoverage"
QT_MOC_LITERAL(97, 1602, 9), // "glScissor"
QT_MOC_LITERAL(98, 1612, 13), // "glStencilFunc"
QT_MOC_LITERAL(99, 1626, 21), // "glStencilFuncSeparate"
QT_MOC_LITERAL(100, 1648, 13), // "glStencilMask"
QT_MOC_LITERAL(101, 1662, 21), // "glStencilMaskSeparate"
QT_MOC_LITERAL(102, 1684, 11), // "glStencilOp"
QT_MOC_LITERAL(103, 1696, 19), // "glStencilOpSeparate"
QT_MOC_LITERAL(104, 1716, 12), // "glTexImage2D"
QT_MOC_LITERAL(105, 1729, 15), // "glTexParameterf"
QT_MOC_LITERAL(106, 1745, 15), // "glTexParameteri"
QT_MOC_LITERAL(107, 1761, 15), // "glTexSubImage2D"
QT_MOC_LITERAL(108, 1777, 11), // "glUniform1f"
QT_MOC_LITERAL(109, 1789, 11), // "glUniform2f"
QT_MOC_LITERAL(110, 1801, 11), // "glUniform3f"
QT_MOC_LITERAL(111, 1813, 11), // "glUniform4f"
QT_MOC_LITERAL(112, 1825, 11), // "glUniform1i"
QT_MOC_LITERAL(113, 1837, 11), // "glUniform2i"
QT_MOC_LITERAL(114, 1849, 11), // "glUniform3i"
QT_MOC_LITERAL(115, 1861, 11), // "glUniform4i"
QT_MOC_LITERAL(116, 1873, 12), // "glUniform1fv"
QT_MOC_LITERAL(117, 1886, 12), // "glUniform2fv"
QT_MOC_LITERAL(118, 1899, 12), // "glUniform3fv"
QT_MOC_LITERAL(119, 1912, 12), // "glUniform4fv"
QT_MOC_LITERAL(120, 1925, 12), // "glUniform1iv"
QT_MOC_LITERAL(121, 1938, 12), // "glUniform2iv"
QT_MOC_LITERAL(122, 1951, 12), // "glUniform3iv"
QT_MOC_LITERAL(123, 1964, 12), // "glUniform4iv"
QT_MOC_LITERAL(124, 1977, 18), // "glUniformMatrix2fv"
QT_MOC_LITERAL(125, 1996, 18), // "glUniformMatrix3fv"
QT_MOC_LITERAL(126, 2015, 18), // "glUniformMatrix4fv"
QT_MOC_LITERAL(127, 2034, 12), // "glUseProgram"
QT_MOC_LITERAL(128, 2047, 17), // "glValidateProgram"
QT_MOC_LITERAL(129, 2065, 16), // "glVertexAttrib1f"
QT_MOC_LITERAL(130, 2082, 16), // "glVertexAttrib2f"
QT_MOC_LITERAL(131, 2099, 16), // "glVertexAttrib3f"
QT_MOC_LITERAL(132, 2116, 16), // "glVertexAttrib4f"
QT_MOC_LITERAL(133, 2133, 17), // "glVertexAttrib1fv"
QT_MOC_LITERAL(134, 2151, 17), // "glVertexAttrib2fv"
QT_MOC_LITERAL(135, 2169, 17), // "glVertexAttrib3fv"
QT_MOC_LITERAL(136, 2187, 17), // "glVertexAttrib4fv"
QT_MOC_LITERAL(137, 2205, 21), // "glVertexAttribPointer"
QT_MOC_LITERAL(138, 2227, 10), // "glViewport"
QT_MOC_LITERAL(139, 2238, 22), // "internalGetUniformType"
QT_MOC_LITERAL(140, 2261, 21), // "internalClearLocation"
QT_MOC_LITERAL(141, 2283, 18), // "internalBeginPaint"
QT_MOC_LITERAL(142, 2302, 23), // "internalTextureComplete"
QT_MOC_LITERAL(143, 2326, 31), // "internalClearQuickItemAsTexture"
QT_MOC_LITERAL(144, 2358, 12) // "extStateDump"

    },
    "QtCanvas3D::CanvasGlCommandQueue\0"
    "queueFull\0\0GlCommandId\0internalNoCommand\0"
    "glActiveTexture\0glAttachShader\0"
    "glBindAttribLocation\0glBindBuffer\0"
    "glBindFramebuffer\0glBindRenderbuffer\0"
    "glBindTexture\0glBlendColor\0glBlendEquation\0"
    "glBlendEquationSeparate\0glBlendFunc\0"
    "glBlendFuncSeparate\0glBufferData\0"
    "glBufferSubData\0glCheckFramebufferStatus\0"
    "glClear\0glClearColor\0glClearDepthf\0"
    "glClearStencil\0glColorMask\0glCompileShader\0"
    "glCompressedTexImage2D\0glCompressedTexSubImage2D\0"
    "glCopyTexImage2D\0glCopyTexSubImage2D\0"
    "glCreateProgram\0glCreateShader\0"
    "glCullFace\0glDeleteBuffers\0"
    "glDeleteFramebuffers\0glDeleteProgram\0"
    "glDeleteRenderbuffers\0glDeleteShader\0"
    "glDeleteTextures\0glDepthFunc\0glDepthMask\0"
    "glDepthRangef\0glDetachShader\0glDisable\0"
    "glDisableVertexAttribArray\0glDrawArrays\0"
    "glDrawElements\0glEnable\0"
    "glEnableVertexAttribArray\0glFlush\0"
    "glFinish\0glFramebufferRenderbuffer\0"
    "glFramebufferTexture2D\0glFrontFace\0"
    "glGenBuffers\0glGenerateMipmap\0"
    "glGenFramebuffers\0glGenRenderbuffers\0"
    "glGenTextures\0glGetActiveAttrib\0"
    "glGetActiveUniform\0glGetAttribLocation\0"
    "glGetBooleanv\0glGetBufferParameteriv\0"
    "glGetError\0glGetFloatv\0"
    "glGetFramebufferAttachmentParameteriv\0"
    "glGetIntegerv\0glGetProgramInfoLog\0"
    "glGetProgramiv\0glGetRenderbufferParameteriv\0"
    "glGetShaderInfoLog\0glGetShaderiv\0"
    "glGetShaderPrecisionFormat\0glGetString\0"
    "glGetTexParameteriv\0glGetUniformfv\0"
    "glGetUniformiv\0glGetUniformLocation\0"
    "glGetVertexAttribPointerv\0glGetVertexAttribfv\0"
    "glGetVertexAttribiv\0glHint\0glIsBuffer\0"
    "glIsEnabled\0glIsFramebuffer\0glIsProgram\0"
    "glIsRenderbuffer\0glIsShader\0glIsTexture\0"
    "glLineWidth\0glLinkProgram\0glPixelStorei\0"
    "glPolygonOffset\0glReadPixels\0"
    "glRenderbufferStorage\0glSampleCoverage\0"
    "glScissor\0glStencilFunc\0glStencilFuncSeparate\0"
    "glStencilMask\0glStencilMaskSeparate\0"
    "glStencilOp\0glStencilOpSeparate\0"
    "glTexImage2D\0glTexParameterf\0"
    "glTexParameteri\0glTexSubImage2D\0"
    "glUniform1f\0glUniform2f\0glUniform3f\0"
    "glUniform4f\0glUniform1i\0glUniform2i\0"
    "glUniform3i\0glUniform4i\0glUniform1fv\0"
    "glUniform2fv\0glUniform3fv\0glUniform4fv\0"
    "glUniform1iv\0glUniform2iv\0glUniform3iv\0"
    "glUniform4iv\0glUniformMatrix2fv\0"
    "glUniformMatrix3fv\0glUniformMatrix4fv\0"
    "glUseProgram\0glValidateProgram\0"
    "glVertexAttrib1f\0glVertexAttrib2f\0"
    "glVertexAttrib3f\0glVertexAttrib4f\0"
    "glVertexAttrib1fv\0glVertexAttrib2fv\0"
    "glVertexAttrib3fv\0glVertexAttrib4fv\0"
    "glVertexAttribPointer\0glViewport\0"
    "internalGetUniformType\0internalClearLocation\0"
    "internalBeginPaint\0internalTextureComplete\0"
    "internalClearQuickItemAsTexture\0"
    "extStateDump"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtCanvas3D__CanvasGlCommandQueue[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       1,   20, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void,

 // enums: name, flags, count, data
       3, 0x0,  141,   24,

 // enum data: key, value
       4, uint(QtCanvas3D::CanvasGlCommandQueue::internalNoCommand),
       5, uint(QtCanvas3D::CanvasGlCommandQueue::glActiveTexture),
       6, uint(QtCanvas3D::CanvasGlCommandQueue::glAttachShader),
       7, uint(QtCanvas3D::CanvasGlCommandQueue::glBindAttribLocation),
       8, uint(QtCanvas3D::CanvasGlCommandQueue::glBindBuffer),
       9, uint(QtCanvas3D::CanvasGlCommandQueue::glBindFramebuffer),
      10, uint(QtCanvas3D::CanvasGlCommandQueue::glBindRenderbuffer),
      11, uint(QtCanvas3D::CanvasGlCommandQueue::glBindTexture),
      12, uint(QtCanvas3D::CanvasGlCommandQueue::glBlendColor),
      13, uint(QtCanvas3D::CanvasGlCommandQueue::glBlendEquation),
      14, uint(QtCanvas3D::CanvasGlCommandQueue::glBlendEquationSeparate),
      15, uint(QtCanvas3D::CanvasGlCommandQueue::glBlendFunc),
      16, uint(QtCanvas3D::CanvasGlCommandQueue::glBlendFuncSeparate),
      17, uint(QtCanvas3D::CanvasGlCommandQueue::glBufferData),
      18, uint(QtCanvas3D::CanvasGlCommandQueue::glBufferSubData),
      19, uint(QtCanvas3D::CanvasGlCommandQueue::glCheckFramebufferStatus),
      20, uint(QtCanvas3D::CanvasGlCommandQueue::glClear),
      21, uint(QtCanvas3D::CanvasGlCommandQueue::glClearColor),
      22, uint(QtCanvas3D::CanvasGlCommandQueue::glClearDepthf),
      23, uint(QtCanvas3D::CanvasGlCommandQueue::glClearStencil),
      24, uint(QtCanvas3D::CanvasGlCommandQueue::glColorMask),
      25, uint(QtCanvas3D::CanvasGlCommandQueue::glCompileShader),
      26, uint(QtCanvas3D::CanvasGlCommandQueue::glCompressedTexImage2D),
      27, uint(QtCanvas3D::CanvasGlCommandQueue::glCompressedTexSubImage2D),
      28, uint(QtCanvas3D::CanvasGlCommandQueue::glCopyTexImage2D),
      29, uint(QtCanvas3D::CanvasGlCommandQueue::glCopyTexSubImage2D),
      30, uint(QtCanvas3D::CanvasGlCommandQueue::glCreateProgram),
      31, uint(QtCanvas3D::CanvasGlCommandQueue::glCreateShader),
      32, uint(QtCanvas3D::CanvasGlCommandQueue::glCullFace),
      33, uint(QtCanvas3D::CanvasGlCommandQueue::glDeleteBuffers),
      34, uint(QtCanvas3D::CanvasGlCommandQueue::glDeleteFramebuffers),
      35, uint(QtCanvas3D::CanvasGlCommandQueue::glDeleteProgram),
      36, uint(QtCanvas3D::CanvasGlCommandQueue::glDeleteRenderbuffers),
      37, uint(QtCanvas3D::CanvasGlCommandQueue::glDeleteShader),
      38, uint(QtCanvas3D::CanvasGlCommandQueue::glDeleteTextures),
      39, uint(QtCanvas3D::CanvasGlCommandQueue::glDepthFunc),
      40, uint(QtCanvas3D::CanvasGlCommandQueue::glDepthMask),
      41, uint(QtCanvas3D::CanvasGlCommandQueue::glDepthRangef),
      42, uint(QtCanvas3D::CanvasGlCommandQueue::glDetachShader),
      43, uint(QtCanvas3D::CanvasGlCommandQueue::glDisable),
      44, uint(QtCanvas3D::CanvasGlCommandQueue::glDisableVertexAttribArray),
      45, uint(QtCanvas3D::CanvasGlCommandQueue::glDrawArrays),
      46, uint(QtCanvas3D::CanvasGlCommandQueue::glDrawElements),
      47, uint(QtCanvas3D::CanvasGlCommandQueue::glEnable),
      48, uint(QtCanvas3D::CanvasGlCommandQueue::glEnableVertexAttribArray),
      49, uint(QtCanvas3D::CanvasGlCommandQueue::glFlush),
      50, uint(QtCanvas3D::CanvasGlCommandQueue::glFinish),
      51, uint(QtCanvas3D::CanvasGlCommandQueue::glFramebufferRenderbuffer),
      52, uint(QtCanvas3D::CanvasGlCommandQueue::glFramebufferTexture2D),
      53, uint(QtCanvas3D::CanvasGlCommandQueue::glFrontFace),
      54, uint(QtCanvas3D::CanvasGlCommandQueue::glGenBuffers),
      55, uint(QtCanvas3D::CanvasGlCommandQueue::glGenerateMipmap),
      56, uint(QtCanvas3D::CanvasGlCommandQueue::glGenFramebuffers),
      57, uint(QtCanvas3D::CanvasGlCommandQueue::glGenRenderbuffers),
      58, uint(QtCanvas3D::CanvasGlCommandQueue::glGenTextures),
      59, uint(QtCanvas3D::CanvasGlCommandQueue::glGetActiveAttrib),
      60, uint(QtCanvas3D::CanvasGlCommandQueue::glGetActiveUniform),
      61, uint(QtCanvas3D::CanvasGlCommandQueue::glGetAttribLocation),
      62, uint(QtCanvas3D::CanvasGlCommandQueue::glGetBooleanv),
      63, uint(QtCanvas3D::CanvasGlCommandQueue::glGetBufferParameteriv),
      64, uint(QtCanvas3D::CanvasGlCommandQueue::glGetError),
      65, uint(QtCanvas3D::CanvasGlCommandQueue::glGetFloatv),
      66, uint(QtCanvas3D::CanvasGlCommandQueue::glGetFramebufferAttachmentParameteriv),
      67, uint(QtCanvas3D::CanvasGlCommandQueue::glGetIntegerv),
      68, uint(QtCanvas3D::CanvasGlCommandQueue::glGetProgramInfoLog),
      69, uint(QtCanvas3D::CanvasGlCommandQueue::glGetProgramiv),
      70, uint(QtCanvas3D::CanvasGlCommandQueue::glGetRenderbufferParameteriv),
      71, uint(QtCanvas3D::CanvasGlCommandQueue::glGetShaderInfoLog),
      72, uint(QtCanvas3D::CanvasGlCommandQueue::glGetShaderiv),
      73, uint(QtCanvas3D::CanvasGlCommandQueue::glGetShaderPrecisionFormat),
      74, uint(QtCanvas3D::CanvasGlCommandQueue::glGetString),
      75, uint(QtCanvas3D::CanvasGlCommandQueue::glGetTexParameteriv),
      76, uint(QtCanvas3D::CanvasGlCommandQueue::glGetUniformfv),
      77, uint(QtCanvas3D::CanvasGlCommandQueue::glGetUniformiv),
      78, uint(QtCanvas3D::CanvasGlCommandQueue::glGetUniformLocation),
      79, uint(QtCanvas3D::CanvasGlCommandQueue::glGetVertexAttribPointerv),
      80, uint(QtCanvas3D::CanvasGlCommandQueue::glGetVertexAttribfv),
      81, uint(QtCanvas3D::CanvasGlCommandQueue::glGetVertexAttribiv),
      82, uint(QtCanvas3D::CanvasGlCommandQueue::glHint),
      83, uint(QtCanvas3D::CanvasGlCommandQueue::glIsBuffer),
      84, uint(QtCanvas3D::CanvasGlCommandQueue::glIsEnabled),
      85, uint(QtCanvas3D::CanvasGlCommandQueue::glIsFramebuffer),
      86, uint(QtCanvas3D::CanvasGlCommandQueue::glIsProgram),
      87, uint(QtCanvas3D::CanvasGlCommandQueue::glIsRenderbuffer),
      88, uint(QtCanvas3D::CanvasGlCommandQueue::glIsShader),
      89, uint(QtCanvas3D::CanvasGlCommandQueue::glIsTexture),
      90, uint(QtCanvas3D::CanvasGlCommandQueue::glLineWidth),
      91, uint(QtCanvas3D::CanvasGlCommandQueue::glLinkProgram),
      92, uint(QtCanvas3D::CanvasGlCommandQueue::glPixelStorei),
      93, uint(QtCanvas3D::CanvasGlCommandQueue::glPolygonOffset),
      94, uint(QtCanvas3D::CanvasGlCommandQueue::glReadPixels),
      95, uint(QtCanvas3D::CanvasGlCommandQueue::glRenderbufferStorage),
      96, uint(QtCanvas3D::CanvasGlCommandQueue::glSampleCoverage),
      97, uint(QtCanvas3D::CanvasGlCommandQueue::glScissor),
      98, uint(QtCanvas3D::CanvasGlCommandQueue::glStencilFunc),
      99, uint(QtCanvas3D::CanvasGlCommandQueue::glStencilFuncSeparate),
     100, uint(QtCanvas3D::CanvasGlCommandQueue::glStencilMask),
     101, uint(QtCanvas3D::CanvasGlCommandQueue::glStencilMaskSeparate),
     102, uint(QtCanvas3D::CanvasGlCommandQueue::glStencilOp),
     103, uint(QtCanvas3D::CanvasGlCommandQueue::glStencilOpSeparate),
     104, uint(QtCanvas3D::CanvasGlCommandQueue::glTexImage2D),
     105, uint(QtCanvas3D::CanvasGlCommandQueue::glTexParameterf),
     106, uint(QtCanvas3D::CanvasGlCommandQueue::glTexParameteri),
     107, uint(QtCanvas3D::CanvasGlCommandQueue::glTexSubImage2D),
     108, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform1f),
     109, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform2f),
     110, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform3f),
     111, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform4f),
     112, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform1i),
     113, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform2i),
     114, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform3i),
     115, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform4i),
     116, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform1fv),
     117, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform2fv),
     118, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform3fv),
     119, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform4fv),
     120, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform1iv),
     121, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform2iv),
     122, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform3iv),
     123, uint(QtCanvas3D::CanvasGlCommandQueue::glUniform4iv),
     124, uint(QtCanvas3D::CanvasGlCommandQueue::glUniformMatrix2fv),
     125, uint(QtCanvas3D::CanvasGlCommandQueue::glUniformMatrix3fv),
     126, uint(QtCanvas3D::CanvasGlCommandQueue::glUniformMatrix4fv),
     127, uint(QtCanvas3D::CanvasGlCommandQueue::glUseProgram),
     128, uint(QtCanvas3D::CanvasGlCommandQueue::glValidateProgram),
     129, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttrib1f),
     130, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttrib2f),
     131, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttrib3f),
     132, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttrib4f),
     133, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttrib1fv),
     134, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttrib2fv),
     135, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttrib3fv),
     136, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttrib4fv),
     137, uint(QtCanvas3D::CanvasGlCommandQueue::glVertexAttribPointer),
     138, uint(QtCanvas3D::CanvasGlCommandQueue::glViewport),
     139, uint(QtCanvas3D::CanvasGlCommandQueue::internalGetUniformType),
     140, uint(QtCanvas3D::CanvasGlCommandQueue::internalClearLocation),
     141, uint(QtCanvas3D::CanvasGlCommandQueue::internalBeginPaint),
     142, uint(QtCanvas3D::CanvasGlCommandQueue::internalTextureComplete),
     143, uint(QtCanvas3D::CanvasGlCommandQueue::internalClearQuickItemAsTexture),
     144, uint(QtCanvas3D::CanvasGlCommandQueue::extStateDump),

       0        // eod
};

void QtCanvas3D::CanvasGlCommandQueue::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        CanvasGlCommandQueue *_t = static_cast<CanvasGlCommandQueue *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->queueFull(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (CanvasGlCommandQueue::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&CanvasGlCommandQueue::queueFull)) {
                *result = 0;
                return;
            }
        }
    }
    Q_UNUSED(_a);
}

const QMetaObject QtCanvas3D::CanvasGlCommandQueue::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_QtCanvas3D__CanvasGlCommandQueue.data,
      qt_meta_data_QtCanvas3D__CanvasGlCommandQueue,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *QtCanvas3D::CanvasGlCommandQueue::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtCanvas3D::CanvasGlCommandQueue::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtCanvas3D__CanvasGlCommandQueue.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtCanvas3D::CanvasGlCommandQueue::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QtCanvas3D::CanvasGlCommandQueue::queueFull()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
